﻿/*
Lith GUI - Main Class
20 July 2008
*/

#include <windows.h>
#include <commctrl.h>
#pragma hdrstop

#include "tlib.h"

#include "CGUI.h"
#include "CForm.h"
#include "IThread.h"

using namespace tlib;
using namespace LithGUI;



CGUI::CGUI(HINSTANCE aInstance)
{
	INITCOMMONCONTROLSEX icc;
	memset(&icc, 0, sizeof(INITCOMMONCONTROLSEX));
	icc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	if (InitCommonControlsEx(&icc) == 0)
		throw TException(L"InitCommonControlsEx() failed");

	mInstance = aInstance;

	mNumClasses = 0;
}



CGUI::~CGUI()
{
	for (int i = 0; i < mThreads.NumItems; i++)
	{
		if (!CloseHandle(mThreads[i]))
			throw TException(L"CloseHandle() failed");
	}
}



CForm* CGUI::CreateForm(IMessageHandler* aMessageHandler, TString aCaption, int aXSize,
		int aYSize)
{
	TString Name = TString::SPrintF(L"%d", mNumClasses);
	mNumClasses++;

	WNDCLASSEX ClassInfo;
	ZeroMemory(&ClassInfo, sizeof(WNDCLASSEX));
	ClassInfo.cbSize = sizeof(WNDCLASSEX);
	ClassInfo.style = CS_OWNDC;
	ClassInfo.lpfnWndProc = WndProc;
	//ClassInfo.cbClsExtra = sizeof(LONG_PTR);
	ClassInfo.hInstance = mInstance;
	ClassInfo.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	ClassInfo.lpszClassName = Name.Str;
	if (RegisterClassEx(&ClassInfo) == 0)
		throw TException(L"RegisterClassEx() failed");

	mClassMessageHandler.Add(Name, aMessageHandler);

	CForm *Form = new CForm(this, Name, aCaption, aXSize, aYSize);
	mFormList.Add(Form);
	return Form;
}



LRESULT CALLBACK CGUI::WndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	CForm *Form;
	TString ClassName;
	if (msg == WM_NCCREATE)
	{
		CREATESTRUCT *cs = (CREATESTRUCT*)lparam;

		TString ClassName(wcslen(cs->lpszClass), cs->lpszClass);

		Form = (CForm*)cs->lpCreateParams;
		SetLastError(0);
		if (SetWindowLongPtr(hwnd, GWLP_USERDATA, (LONG_PTR)Form) == 0 && GetLastError() != 0)
			throw TException(L"SetWindowLongPtr() failed");
	}
	else
	{
		wchar_t ClassNameStr[13];
		GetClassName(hwnd, ClassNameStr, 13);
		TString ClassName(wcslen(ClassNameStr), ClassNameStr);

		SetLastError(0);
		LONG_PTR Ptr = GetWindowLongPtr(hwnd, GWLP_USERDATA);
		if (Ptr == 0 && GetLastError() != 0)
			throw TException(L"GetWindowLongPtr() failed");
		Form = (CForm*)Ptr;
		if (Form == NULL)   // process messages after destroying window
			return DefWindowProc(hwnd, msg, wparam, lparam);
	}

	LRESULT Result;
	bool Handled = Form->GetOwner()->HandleMessage(Form, msg, wparam, lparam, Result);
	if (!Handled)
		Result = DefWindowProc(hwnd, msg, wparam, lparam);
	return Result;
}



bool CGUI::HandleMessage(CForm* aForm, UINT aMessage, WPARAM aWParam, LPARAM aLParam,
		LRESULT& aResult)
{
	IMessageHandler *MessageHandler = mClassMessageHandler[aForm->GetClassName()];

	LRESULT Result = 0;
	bool Handled = MessageHandler->Message(aMessage, aWParam, aLParam, aResult);

	switch (aMessage)
	{
	case WM_DESTROY:
		Handled = true;
		aResult = 0;
		SetLastError(0);
		if (!SetWindowLongPtr(aForm->GetWindow(), GWLP_USERDATA, NULL) && GetLastError() != 0)
			throw TException(L"SetWindowLongPtr() failed");
		mFormList.RemoveItem(aForm);
		delete aForm;
		break;
	}
	return Handled;
}



void CGUI::CreateThread(IThread* aThread)
{
	DWORD ThreadID;
	HANDLE Thread = ::CreateThread(NULL, 0, RunThread, aThread, 0, &ThreadID);
	if (Thread == NULL)
		throw TException(L"CreateThread() failed");
	mThreads.Add(Thread);
}



DWORD WINAPI CGUI::RunThread(LPVOID lpParameter)
{
	IThread *Thread = (IThread*)lpParameter;
	Thread->RunThread();
	return 0;
}