﻿#include <windows.h>
#pragma hdrstop

#include "tlib.h"

#include "CBitmap.h"
#include "CFile.h"

using namespace tlib;
using namespace LithUtil;



CBitmap::CBitmap(TString aFilePath)
{
	HGDIOBJ hBitmap = (HGDIOBJ)LoadImage(NULL, aFilePath.Str, IMAGE_BITMAP, 0, 0,
			LR_LOADFROMFILE | LR_CREATEDIBSECTION);
	if (hBitmap == NULL)
		throw TException(L"LoadImage() failed");
	BITMAP Bitmap;
	if (GetObject(hBitmap, sizeof(BITMAP), &Bitmap) == 0)
		throw TException(L"GetObject() failed");
	if (!(Bitmap.bmBitsPixel == 24 || Bitmap.bmBitsPixel == 32))
	{
		DeleteObject(hBitmap);
		throw TException(L"Unsupported bit depth");
	}
	mXSize = Bitmap.bmWidth;
	mYSize = Bitmap.bmHeight;
	mData = new unsigned int [mXSize * mYSize];
	for (int y = 0; y < mYSize; y++)
	{
		int Offset = Bitmap.bmWidthBytes * (mYSize - y - 1);
		for (int x = 0; x < mXSize; x++)
		{
			unsigned int Colour;
			if (Bitmap.bmBitsPixel == 24)
			{
				unsigned char R = ((TByte*)Bitmap.bmBits)[Offset + x * 3 + 2];
				unsigned char G = ((TByte*)Bitmap.bmBits)[Offset + x * 3 + 1];
				unsigned char B = ((TByte*)Bitmap.bmBits)[Offset + x * 3 + 0];
				Colour = (R << 16) | (G << 8) | B;
			}
			else   // 32-bit TODO: actually test this
				Colour = *(unsigned int*)(((TByte*)Bitmap.bmBits) + Offset + x * 4) & 0xffffff;
			if (Colour == 0xffffff)   // white: transparent
				mData[y * mXSize + x] = 0x00000000;
			else if (Colour == 0xff00ff)   // magenta: semi-transparent
				mData[y * mXSize + x] = 0x7f7f7f7f;
			else
				mData[y * mXSize + x] = 0xff000000 | Colour;
		}
	}
	DeleteObject(hBitmap);
}



CBitmap::~CBitmap()
{
	delete [] mData;
}